@@ -32,6 +32,7 @@ utils::globalVariables(c(
   "data_train",
   "textTrainCVpredictions", "textTrainCVpredictionsRF",
   "strata",
+  "data_prepared_with_recipe",
 
   # textTrainNPlot
   "sample_size", "percent", "std",
@@ -86,6 +87,7 @@ utils::globalVariables(c(
   "topics_corr_grouping", "extract_topic_stats_corr",
   "adjust.p_value", "mean_group_1", "mean_group_2", "sort_stats_tibble", "label_1", "top_terms",
   "topics_t_test_grouping", "extract_topic_stats_cate", "topic_name", "p.value",
-  "adjusted_p.value", "cohen_d", "prevalence", "coherence"
+  "adjusted_p.value", "cohen_d", "prevalence", "coherence", "log_sample_size", "reduce_topics",
+  "get_topic_tree"
 
 ))
