@@ -1,13 +1,14 @@
-
-
 #' Zero Shot Classification (Experimental)
-#' @param sequences (string)  The sequence(s) to classify (not that they will be truncated if the model input is too large).
-#' @param candidate_labels (string) The set of class labels that is possible in the to classification of each sequence.
-#' It may be a single label, a string of comma-separated labels, or a list of labels.
+#' @param sequences (string)  The sequence(s) to classify (not that they will be truncated
+#' if the model input is too large).
+#' @param candidate_labels (string) The set of class labels that is possible in the to
+#' classification of each sequence. It may be a single label, a string of comma-separated
+#' labels, or a list of labels.
 #' @param hypothesis_template (string; optional)
 #' The template that is used for turning each of the label into an NLI-style hypothesis.
-#' This template must include a "{}" or similar syntax so that the candidate label can be inserted into the template.
-#' For example, the default template is "This example is {}." With the candidate label "sports",
+#' This template must include a "{}" or similar syntax so that the candidate label can be
+#' inserted into the template. For example, the default template is
+#' "This example is {}." With the candidate label "sports",
 #' this would be fed into the model like "<cls> sequence to classify <sep> This example is sports . <sep>".
 #' The default template works well in many cases, but it may be worthwhile to experiment with different templates
 #' depending on the task setting (see https://huggingface.co/docs/transformers/).
@@ -55,21 +56,22 @@ textZeroShot <- function(sequences,
   # Run python file with HunggingFace interface to state-of-the-art transformers
   reticulate::source_python(system.file("python",
     "huggingface_Interface3.py",
-    # envir = NULL,
     package = "text",
     mustWork = TRUE
   ))
 
-  hg_zeroshot <- hgTransformerGetZeroShot(sequences = sequences,
-                                          candidate_labels = candidate_labels,
-                                          hypothesis_template = hypothesis_template,
-                                          multi_label = multi_label,
-                                          model = model,
-                                          device = device,
-                                          tokenizer_parallelism = tokenizer_parallelism,
-                                          logging_level = logging_level,
-                                          return_incorrect_results = return_incorrect_results,
-                                          set_seed = set_seed)
+  hg_zeroshot <- hgTransformerGetZeroShot(
+    sequences = sequences,
+    candidate_labels = candidate_labels,
+    hypothesis_template = hypothesis_template,
+    multi_label = multi_label,
+    model = model,
+    device = device,
+    tokenizer_parallelism = tokenizer_parallelism,
+    logging_level = logging_level,
+    return_incorrect_results = return_incorrect_results,
+    set_seed = set_seed
+  )
 
   T1_text_all <- Sys.time()
 
@@ -81,17 +83,20 @@ textZeroShot <- function(sequences,
     ) %>%
     dplyr::ungroup() %>%
     tidyr::pivot_wider(
-      id_cols= sequence,
+      id_cols = sequence,
       names_from = no,
       names_prefix = "x_",
       values_from = c(labels, scores),
-      names_sort=TRUE
+      names_sort = TRUE
     )
 
   # Re-arrange to mix order of labels and scores. help(pivot_wider)
-  hg_zeroshot1 <- cbind(hg_zeroshot1[1],
-                        hg_zeroshot1[c(matrix(names(hg_zeroshot1)[-1], 2,
-                                              byrow = TRUE))])
+  hg_zeroshot1 <- cbind(
+    hg_zeroshot1[1],
+    hg_zeroshot1[c(matrix(names(hg_zeroshot1)[-1], 2,
+                          byrow = TRUE
+    ))]
+  )
 
 
   # Time to complete all variables
@@ -114,7 +119,3 @@ textZeroShot <- function(sequences,
   )
   return(hg_zeroshot1)
 }
-
-
-
-
